/******************************************************************************
* Module	:	Maze Implementation
*
* Author	:	John W. M. Stevens
******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<time.h>

#include	<objc/Object.h>

#include	"CharacterMaze.h"

@implementation		CharacterMaze	:	Maze

- init
{
	/*  Initialize data attributes. */
	xMaximum = 79;
	yMaximum = 79;
	xCount   = 39;
	yCount   = 39;
	offset   = 0;

	/*	Initialize the super class.	*/
	[ super init ];

	/*  Initialize maze to have no openings.    */
	memset(
		buffer,
		' ',
		sizeof ( uchar_t ) * 80 * 80 );

	/*	Return self.	*/
	return self;
}

- initWithXDimension: (int) inXCount
		  YDimension: (int) inYCount
{
	/*  Initialize data attributes. */
	xMaximum = 79;
	yMaximum = 79;
	xCount   = inXCount;
	yCount   = inYCount;
	offset   = 0;

	/*	Initialize the super class.	*/
	[ super init ];

	/*  Initialize maze to have no openings.    */
	memset(
		buffer,
		' ',
		sizeof ( uchar_t ) * 80 * 80 );

	/*	Return self.	*/
	return self;
}

- drawLineXStart: (int) x
		  YStart: (int) y
		    XEnd: (int) xEnd
		    YEnd: (int) yEnd
{
	register	int		i;
	register	int		step;
	auto		int		swap;

	/*	Draw line into buffer.	*/
	if (x == xEnd)
	{
		/*	Calculate the step.	*/
		if (y > yEnd)
		{
			swap = yEnd;
			yEnd = y;
			y    = swap;
		}

		/*	Draw vertical line.	*/
		for (i = y * 2; i < yEnd * 2; i += 2)
		{
			buffer[ 2 * x ][ i     ] = '+';
			buffer[ 2 * x ][ i + 1 ] = '|';
		}
		buffer[ 2 * x ][ i ] = '+';
	}
	else
	{
		/*	Calculate the step.	*/
		if (x > xEnd)
		{
			swap = xEnd;
			xEnd = x;
			x    = swap;
		}

		/*	Draw vertical line.	*/
		for (i = x * 2; i < xEnd * 2; i += 2)
		{
			buffer[ i     ][ 2 * y ] = '+';
			buffer[ i + 1 ][ 2 * y ] = '-';
		}
		buffer[ i ][ 2 * y ] = '+';
	}

	return self;
}

- drawSolutionLineXStart: (int) xPrevious
				  YStart: (int) yPrevious
					XEnd: (int) x
					YEnd: (int) y
{
	auto	int		diff;

	buffer[ xPrevious * 2 + 1 ][ yPrevious * 2 + 1 ] = '@';
	buffer[ x         * 2 + 1 ][ y         * 2 + 1 ] = '@';

	if (x == xPrevious)
	{
		y =
			(yPrevious < y)
				?	2 * y
				:	2 * yPrevious ;
		buffer[ x * 2 + 1 ][ y ] = '@';
	}
	else
	{
		x =
			(xPrevious < x)
				?	2 * x
				:	2 * xPrevious ;
		buffer[ x ][ y * 2 + 1 ] = '@';
	}

	return self;
}

- endPage
{
	register	int		i;
	register	int		j;

	[ super beginPage ];

	/*	Print character buffer.	*/
	for (i = 0; i < yCount * 2 + 1; i++)
	{
		for (j = 0; j < xCount * 2 + 1; j++)
			fputc(
				buffer[ j ][ i ],
				stdout );
		fputc(
			'\n',
			stdout );
	}
	fputc(
		'\n',
		stdout );

	return self;
}

@end
