/******************************************************************************
* Module	:	LaTeX Maze Implementation
*
* Author	:	John W. M. Stevens
******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<time.h>

#include	<objc/Object.h>

#include	"LaTeXMaze.h"

@implementation		LaTeXMaze	:	Maze

- init
{
	/*  Initialize data attributes. */
	xMaximum   = 460;
	yMaximum   = 640;
	xCount     = 30;
	yCount     = 30;
	offset     = 0;
	pageNumber = 0;

	/*	Initialize the super class.	*/
	[ super init ];

	/*	Return self.	*/
	return self;
}

- initWithXDimension: (int) inXCount
		  YDimension: (int) inYCount
{
	/*  Initialize data attributes. */
	xMaximum   = 460;
	yMaximum   = 640;
	xCount     = inXCount;
	yCount     = inYCount;
	offset     = 0;
	pageNumber = 0;

	/*	Initialize the super class.	*/
	[ super init ];

	/*	Return self.	*/
	return self;
}

- drawLineXStart: (int) x
		  YStart: (int) y
		    XEnd: (int) xEnd
		    YEnd: (int) yEnd
{
	auto	int		length;
	auto	int		slope;

	if (y == yEnd)
	{
		if (x < xEnd)
		{
			length = xEnd - x;
			slope  = 1;
		}
		else
		{
			length = x - xEnd;
			slope  = -1;
		}

		printf(
			"\\put(%d, %d){\\line(%d, 0){%d}}\n",
			x * unitLength,
			(yCount - y) * unitLength,
			slope,
			length * unitLength);
	}
	else
	{
		if (y < yEnd)
		{
			length = yEnd - y;
			slope  = -1;
		}
		else
		{
			length = y - yEnd;
			slope  = 1;
		}


		printf(
			"\\put(%d, %d){\\line(0, %d){%d}}\n",
			x * unitLength,
			(yCount - y) * unitLength,
			slope,
			length * unitLength );
	}

	return self;
}

- drawSolutionLineXStart: (int) xPrevious
				  YStart: (int) yPrevious
					XEnd: (int) x
					YEnd: (int) y
{
	auto	int		center  = unitLength >> 1;
	auto	int		length;
	auto	int		slope;

	if (y == yPrevious)
	{
		if (x < xPrevious)
		{
			length = xPrevious - x;
			slope  = 1;
		}
		else
		{
			length = x - xPrevious;
			slope  = -1;
		}

		printf(
			"\\put(%d, %d){\\line(%d, 0){%d}}\n",
			x * unitLength + center,
			(yCount - y) * unitLength - center,
			slope,
			length * unitLength);
	}
	else
	{
		if (y < yPrevious)
		{
			length = yPrevious - y;
			slope  = -1;
		}
		else
		{
			length = y - yPrevious;
			slope  = 1;
		}

		printf(
			"\\put(%d, %d){\\line(0, %d){%d}}\n",
			x * unitLength + center,
			(yCount - y) * unitLength - center,
			slope,
			length * unitLength );
	}

	return self;
}

- printMaze
{
	/*	Print postscript start up.	*/
	printf(
		"\\color{black}\n" );

	/*	Print maze solution.	*/
	[ super
		printMaze ];

	return self;
}

- printMazeSolution
{
	/*	Print postscript start up.	*/
	printf(
		"\\color{red}\n" );

	/*	Print maze solution.	*/
	[ super
		printMazeSolution ];

	return self;
}

- beginDocument
{
	[ super beginDocument ];

	printf(
		"\\documentclass{article}\n"
		"\\usepackage{color}\n"
		"\\topmargin 0pt\n"
		"\\advance \\topmargin by -\\headheight\n"
		"\\advance \\topmargin by -\\headsep\n"
		"\\textheight 8.9in\n"
		"\\oddsidemargin 0pt\n"
		"\\evensidemargin \\oddsidemargin\n"
		"\\marginparwidth 0.5in\n"
		"\\textwidth 6.5in\n"
		"\\pagestyle{empty}\n"
		"\\begin{document}\n"
		"\\setlength{\\unitlength}{1pt}\n" );

	return self;
}


- beginPage
{
	[ super beginPage ];

	printf(
		"\\begin{picture}(%d,%d)\n",
		xMaximum,
		yMaximum );

	return self;
}

- endPage
{
	[ super endPage ];

	printf(
		"\\end{picture}\n"
		"\\newpage\n" );

	return self;
}

- endDocument
{
	[ super endDocument ];

	printf(
		"\\end{document}\n" );

	return self;
}

@end
