/******************************************************************************
* Module	:	Maze Implementation
*
* Author	:	John W. M. Stevens
******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<time.h>

#include	<objc/Object.h>

#include	"Maze.h"

@implementation		Maze	:	Object

- init
{
	/*	Initialize the super class.	*/
	[ super init ];

	/*  Initialize maze to have no openings.    */
	memset(
		maze,
		0,
		sizeof ( uchar_t ) * X_MAXIMUM * Y_MAXIMUM );

	/*  Calculate unit length.  */
	unitLength = yMaximum / yCount;
	if (xMaximum / xCount < unitLength)
		unitLength = xMaximum / xCount;

	/*	Return self.	*/
	return self;
}

- initWithXDimension: (int) inXCount
		  YDimension: (int) inYCount
{
	/*	Initialize the super class.	*/
	[ super init ];

	/*  Initialize maze to have no openings.    */
	memset(
		maze,
		0,
		sizeof ( uchar_t ) * X_MAXIMUM * Y_MAXIMUM );

	/*  Calculate unit length.  */
	unitLength = yMaximum / yCount;
	if (xMaximum / xCount < unitLength)
		unitLength = xMaximum / xCount;

	/*	Return self.	*/
	return self;
}

- drawLineXStart: (int) x
		  YStart: (int) y
		    XEnd: (int) xEnd
		    YEnd: (int) yEnd
{
	return self;
}

- drawSolutionLineXStart: (int) xPrevious
				  YStart: (int) yPrevious
					XEnd: (int) x
					YEnd: (int) y
{
	return self;
}

- createMazeAtX: (int) x
			AtY: (int) y
{
    /*	Try directions at random till all directions have been tried.	*/
	do
	{
		/*	Pick a random direction.	*/
		switch (
			rand(
				) % 4 )
		{
		case 0:
			/*	Mark up direction as having been tried.	*/
			maze[ x ][ y ] |= GO_UP;

			/*	If there is a cell above this one and that cell has not
			*	yet been visited, then try it.
			*/
			if (y > 0 && (maze[ x ][ y - 1 ] & ALL) == 0)
			{
				/*	Put doors in both cells.	*/
				maze[ x ][ y     ] |= UP;
				maze[ x ][ y - 1 ] |= DOWN;

				/*	Call recursively.	*/
				[ self
					createMazeAtX: x
							  AtY: y - 1 ];
			}
			break;
		case 1:
			/*	Mark right direction as having been tried.	*/
			maze[ x ][ y ] |= GO_RIGHT;

			/*	If there is a cell to the right and that cell has not
			*	yet been visited, then try it.
			*/
			if (x < xCount - 1 && (maze[ x + 1 ][ y ] & ALL) == 0)
			{
				/*	Put doors in both cells.	*/
				maze[ x     ][ y ] |= RIGHT;
				maze[ x + 1 ][ y ] |= LEFT;

				/*	Call recursively.	*/
				[ self
					createMazeAtX: x + 1
							  AtY: y     ];
			}
			break;
		case 2:
			/*	Mark down direction as having been tried.	*/
			maze[ x ][ y ] |= GO_DOWN;

			/*	If there is a cell below this one and that cell has not
			*	yet been visited, then try it.
			*/
			if (y < yCount - 1 && (maze[ x ][ y + 1 ] & ALL) == 0)
			{
				/*	Put doors in both cells.	*/
				maze[ x ][ y     ] |= DOWN;
				maze[ x ][ y + 1 ] |= UP;

				/*	Call recursively.	*/
				[ self
					createMazeAtX: x
							  AtY: y + 1 ];
			}
			break;
		case 3:
			/*	Mark left direction as having been tried.	*/
			maze[ x ][ y ] |= GO_LEFT;

			/*	If there is a cell to the left and that cell has not
			*	yet been visited, then try it.
			*/
			if (x > 0 && (maze[ x - 1 ][ y ] & ALL) == 0)
			{
				/*	Put doors in both cells.	*/
				maze[ x     ][ y ] |= LEFT;
				maze[ x - 1 ][ y ] |= RIGHT;

				/*	Call recursively.	*/
				[ self
					createMazeAtX: x - 1
							  AtY: y     ];
			}
			break;
		}

	} while ((maze[ x ][ y ] & TRY_ALL) != TRY_ALL);

	return self;
}

- createMaze
{
	return
		[ self
			createMazeAtX: rand() % xCount
					  AtY: rand() % yCount ];
}

- (int) findSolutionAtX: (int) x
					AtY: (int) y
{
	register	int		i;
	register	int		j;
	register	int		k;
	auto		int		result;

	/*	Been here before?	*/
	k = (y * xCount) + x;
	i = k >> 3;
	j = 1 << (k &  7);
	if (breadCrumbs[ i ] & j)
		/*	Yes.  Return.	*/
		return 0;

	/*	Mark this cell as having been visited.	*/
	breadCrumbs[ i ] |= j;

	/*	Did we find path out?	*/
	if (x == xCount - 1 && y == yCount - 1)
		return 1;

	/*	Try all directions.	*/
	for (i = 1; i < 16; i <<= 1)
	{
		/*	Can we go in the indicated direction?	*/
		if ( maze[ x ][ y ] & i )
		{
			/*	Yes.  Go that direction.	*/
			switch ( i )
			{
			case UP:
				result =
					(y > 0)
						?	[ self
								findSolutionAtX: x
											AtY: y - 1 ]
						:	0;
				break;
			case DOWN:
				result =
					(y < yCount - 1)
						?	[ self
								findSolutionAtX: x
											AtY: y + 1 ]
						:	0;
				break;
			case LEFT:
				result =
					(x > 0)
						?	[ self
								findSolutionAtX: x - 1
											AtY: y     ]
						:	0;
				break;
			case RIGHT:
				result =
					(x < xCount - 1)
						?	[ self
								findSolutionAtX: x + 1
											AtY: y     ]
						:	0;
				break;
			default:
				result = 0;
				break;
			}

			/*	If we found the path out, return.	*/
			if ( result )
			{
				/*	Mark path.	*/
				maze[ x ][ y ] |= i << 4;
				return result;
			}
		}
	}

	/*	Did not find path out.	*/
	return 0;
}

- solveMaze
{
	register	int		i;
	register	int		j;

	/*	Allocate bread crumb trail.	*/
	i           = (yCount * xCount + 8) >> 3;
	breadCrumbs =
		calloc(
			1,
			i );

	/*	Clear build information.	*/
	for (i = 0; i < yCount; i++)
		for (j = 0; j < xCount; j++)
			maze[ j ][ i ] &= 0xf;

	/*	Solve maze.	*/
	[ self
		findSolutionAtX: 0
					AtY: 0 ];

	/*	Deallocate bread crumb trail.	*/
	free(
		breadCrumbs );

	return self;
}

- printMaze
{
	register	int		i;
	register	int		j;
	register	int		k;

	/*	Create entrance and exit.	*/
	maze[ 0          ][ 0          ] |= UP;
	maze[ xCount - 1 ][ yCount - 1 ] |= DOWN;

	/*	Do horizontal lines from top to bottom.	*/
	for (i = 0; i < yCount; i++)
	{
		/*	Do top of cells.	*/
		for (j = 0; j < xCount; )
		{
			/*	Find run to draw line.	*/
			for (k = j; k < xCount; k++)
				if (maze[ k ][ i ] & UP)
					break;

			/*	If there is no run, simply skip.	*/
			if (k == j)
			{
				j++;
				continue;
			}

			/*	Print line.	*/
			[ self
				drawLineXStart: j
						YStart: i
						  XEnd: k
						  YEnd: i ];

			/*	Jump ahead.	*/
			j = k;
		}
	}

	/*	Do bottom of last row of cells.	*/
	for (j = 0; j < xCount; )
	{
		/*	Find run to draw horizontal line.	*/
		for (k = j; k < xCount; k++)
			if (maze[ k ][ yCount - 1 ] & DOWN)
				break;

		/*	If there is no run, simply skip.	*/
		if (k == j)
		{
			j++;
			continue;
		}

		/*	Print line.	*/
		[ self
			drawLineXStart: j
					YStart: yCount
					  XEnd: k
					  YEnd: yCount ];

		/*	Jump ahead.	*/
		j = k;
	}

	/*	Print vertical lines.	*/
	for (j = 0; j < xCount; j++)
	{
		/*	Do left edge of cells.	*/
		for (i = 0; i < yCount; )
		{
			/*	Find run to draw line.	*/
			for (k = i; k < yCount; k++)
				if (maze[ j ][ k ] & LEFT)
					break;

			/*	If there is no run, simply skip.	*/
			if (k == i)
			{
				i++;
				continue;
			}

			/*	Print line.	*/
			[ self
				drawLineXStart: j
						YStart: i
						  XEnd: j
						  YEnd: k ];

			/*	Jump ahead.	*/
			i = k;
		}
	}

	/*	Do right hand of last row of cells.	*/
	for (i = 0; i < yCount; )
	{
		/*	Find run to draw line.	*/
		for (k = i; k < yCount; k++)
			if (maze[ xCount - 1 ][ k ] & RIGHT)
				break;

		/*	If there is no run, simply skip.	*/
		if (k == i)
		{
			i++;
			continue;
		}

		/*	Print line.	*/
		[ self
			drawLineXStart: xCount
					YStart: i
					  XEnd: xCount
					  YEnd: k      ];

		/*	Jump ahead.	*/
		i = k;
	}

	return self;
}

- printMazeSolution
{
	auto	int		x         = 0;
	auto	int		y         = 0;
	auto	int		xPrevious = x;
	auto	int		yPrevious = y;

	/*	Trace out solution path, drawing lines as we go.	*/
	do
	{
		/*	Move in appropriate direction.	*/
		switch ( maze[ x ][ y ] & TRY_ALL )
		{
		case GO_UP:
			y--;
			break;
		case GO_DOWN:
			y++;
			break;
		case GO_RIGHT:
			x++;
			break;
		case GO_LEFT:
			x--;
			break;
		}

		/*	Print line.	*/
		[ self
			drawSolutionLineXStart: xPrevious
							YStart: yPrevious
							  XEnd: x
							  YEnd: y         ];

		/*	Save as previous.	*/
		xPrevious = x;
		yPrevious = y;

	/*	We done?	*/
	} while (x != xCount - 1 || y != yCount - 1);

	return self;
}

- beginDocument
{
	return self;
}

- endDocument
{
	return self;
}

- beginPage
{
	return self;
}

- endPage
{
	return self;
}

@end
