#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<time.h>

#include	<objc/Object.h>

#include	"CharacterMaze.h"
#include	"LaTeXMaze.h"
#include	"PostscriptMaze.h"
#include	"XMaze.h"

int
main(
	int		argc,
	char	**argv )
{
	auto	int		xCount;
	auto	int		yCount;
	auto	char	printer;
	auto	id		maze;

	/*	Get current time and use it as a random seed.	*/
	srand(
		(time_t)
		time(
			(time_t *) 0 ) );

	/*	Check command line syntax.	*/
	if (argc < 3)
	{
		fprintf(
			stderr,
			"Syntax is 'maze <X max> <Y max> [printer]'\n");
		exit(
			1 );
	}

	/*	Get dimensions of the maze from the command line.	*/
	sscanf(
		argv[ 1 ],
		"%d",
		&xCount );
	sscanf(
		argv[ 2 ],
		"%d",
		&yCount );

	/*	What output format?	*/
	if (argc >= 4)
		printer = argv[ 3 ][ 0 ];
	else
		printer = 'L';

	/*	Check legality of dimensions.	*/
	if (yCount > Y_MAXIMUM || yCount < 1 ||
		xCount > X_MAXIMUM || xCount < 1 )
	{
		fprintf(
			stderr,
			"1 <= x <= %d, 1 <= y <= %d\n",
			X_MAXIMUM,
			Y_MAXIMUM );
		exit(
			1 );
	}

	/*	Create a maze object.	*/
	switch ( printer )
	{
	case 'L':
	case 'l':
		maze =
			[ [ LaTeXMaze alloc ]
				initWithXDimension: xCount
						YDimension: yCount ];
		break;
	case 'C':
	case 'c':
		maze =
			[ [ CharacterMaze alloc ]
				initWithXDimension: xCount
						YDimension: yCount ];
		break;
	case 'P':
	case 'p':
		maze =
			[ [ PostscriptMaze alloc ]
				initWithXDimension: xCount
						YDimension: yCount ];
		break;
	case 'X':
	case 'x':
		maze =
			[ [ XMaze alloc ]
				initWithXDimension: xCount
						YDimension: yCount ];
		break;
	default:
		fprintf(
			stderr,
			"%s %d : Error - unrecognized output type.\n",
			__FILE__,
			__LINE__);
		exit(
			1 );
	}

	/*	Create maze and begin document.	*/
	[ maze createMaze    ];
	[ maze beginDocument ];

	/*	Print maze.	*/
	[ maze beginPage     ];
	[ maze printMaze     ];
	[ maze endPage       ];

	/*	Solve maze.	*/
	[ maze solveMaze         ];

	/*	Print maze with solution.	*/
	[ maze beginPage         ];
	[ maze printMaze         ];
	[ maze printMazeSolution ];
	[ maze endPage           ];

	/*	End document.	*/
	[ maze endDocument ];
}
