/******************************************************************************
* Module	:	X Windows Maze Implementation
*
* Author	:	John W. M. Stevens
******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<time.h>

#include	<objc/Object.h>

#include	"XMaze.h"

@implementation		XMaze	:	Maze

- init
{
	/*  Initialize data attributes. */
	xMaximum = 620;
	yMaximum = 460;
	xCount   = 40;
	yCount   = 40;
	offset   = 10;
	isSolved = 0;

	/*	Initialize the super class.	*/
	[ super init ];

	/*	Return self.	*/
	return self;
}

- initWithXDimension: (int) inXCount
		  YDimension: (int) inYCount
{
	/*  Initialize data attributes. */
	xMaximum = 620;
	yMaximum = 460;
	xCount   = inXCount;
	yCount   = inYCount;
	offset   = 10;
	isSolved = 0;

	/*	Initialize the super class.	*/
	[ super init ];

	/*	Return self.	*/
	return self;
}

- drawLineXStart: (int) x
		  YStart: (int) y
		    XEnd: (int) xEnd
		    YEnd: (int) yEnd
{
	XDrawLine(
		xServer,
		window,
		graphicsContext,
		x    * unitLength + offset,
		y    * unitLength + offset,
		xEnd * unitLength + offset,
		yEnd * unitLength + offset );

	return self;
}

- drawSolutionLineXStart: (int) xPrevious
				  YStart: (int) yPrevious
					XEnd: (int) x
					YEnd: (int) y
{
	auto	int		center  = unitLength >> 1;

	XDrawLine(
		xServer,
		window,
		graphicsContext,
		x         * unitLength + offset + center,
		y         * unitLength + offset + center,
		xPrevious * unitLength + offset + center,
		yPrevious * unitLength + offset + center );

	return self;
}

- solveMaze
{
	/*	Solve the maze.	*/
	[ super
		solveMaze ];

	/*	Set solved flag.	*/
	isSolved = 1;

	return self;
}

- printMaze
{
	/*	Set foreground value.	*/
	contextValues.foreground = black.pixel;
	contextValues.background = white.pixel;

	/*	Change GC foreground value.	*/
	XChangeGC(
		xServer,
		graphicsContext,
		GCForeground | GCBackground,
		&contextValues );

	/*	Print maze solution.	*/
	[ super
		printMaze ];

	return self;
}

- printMazeSolution
{
	/*	Set foreground value.	*/
	contextValues.foreground = red.pixel;

	/*	Change GC foreground value.	*/
	XChangeGC(
		xServer,
		graphicsContext,
		GCForeground,
		&contextValues );

	/*	Print maze solution.	*/
	[ super
		printMazeSolution ];

	return self;
}

- beginDocument
{
	auto	XColor	close;

	[ super beginDocument ];

	/*	Open display, get root window.	*/
	xServer    =
		XOpenDisplay(
			NULL );
	rootWindow =
		XRootWindow(
			xServer,
			0 );

	/*	Get color map, allocate colors.	*/
	colorMap =
		XDefaultColormap(
			xServer,
			0 );
	XAllocNamedColor(
		xServer,
		colorMap,
		"black",
		&black,
		&close );
	XAllocNamedColor(
		xServer,
		colorMap,
		"red",
		&red,
		&close );
	XAllocNamedColor(
		xServer,
		colorMap,
		"white",
		&white,
		&close );

	/*	Create and clear the window.	*/
	window =
		XCreateSimpleWindow(
			xServer,
			rootWindow,
			0,
			0,
			xMaximum + offset * 2,
			yMaximum + offset * 2,
			0,
			0,
			white.pixel );

	/*	Select for input events.	*/
	XSelectInput(
		xServer,
		window,
		StructureNotifyMask |
			KeyPressMask    |
			ExposureMask    |
			ButtonPressMask );

	/*	Create a graphics context for drawing.	*/
	contextValues.foreground = black.pixel;
	contextValues.background = white.pixel;
	contextValues.line_width = 1;
	graphicsContext          =
		XCreateGC(
			xServer,
			rootWindow,
			GCForeground | GCBackground | GCLineWidth,
			&contextValues );

	/*	Map window.	*/
	XMapWindow(	
		xServer,
		window );

	return self;
}

- beginPage
{
	[ super beginPage ];

	XClearArea(
		xServer,
		window,
		0,
		0,
		0,
		0,
		True );

	return self;
}

static
Bool
checkEvent(
	Display		*xServer,
	XEvent		*xEvent,
	char		*argument )
{
	if (xEvent->type == Expose)
		return True;

	return False;
}

- endPage
{
	auto	XEvent	xEvent;

	/*	Get and process X events.	*/
	for ( ; ; )
	{
		/*	Get an event.	*/
		XNextEvent(
			xServer,
			&xEvent);

		/*	Process an event.	*/
		if (xEvent.type == Expose && xEvent.xexpose.count == 0)
		{
			if (
				XCheckIfEvent(
					xServer,
					&xEvent,
					checkEvent,
					0 ) == False )
			{
				/*	Redraw the maze.	*/
				[ self
					printMaze ];

				/*	If is solved, redraw solution.	*/
				if ( isSolved )
					[ self
						printMazeSolution ];
			}
		}
		else if (xEvent.type == ConfigureNotify)
		{
			/*	Get new height and width.	*/
			xMaximum = xEvent.xconfigure.width  - 2 * offset;
			yMaximum = xEvent.xconfigure.height - 2 * offset;

			/*  Calculate unit length.  */
			unitLength = yMaximum / yCount;
			if (xMaximum / xCount < unitLength)
				unitLength = xMaximum / xCount;

			/*	Clear the window.	*/
			XClearArea(
				xServer,
				window,
				0,
				0,
				0,
				0,
				True );
		}
		else if (xEvent.type == ButtonPress ||
				 xEvent.type == KeyPress    )
			break;
	}

	return self;
}

- endDocument
{
	[ super endDocument ];

	/*	Unmap the window and destroy it.	*/
	XUnmapWindow(
		xServer,
		window );
	XDestroyWindow(
		xServer,
		window );
	XCloseDisplay(
		xServer );

	return self;
}

@end
